/**
 * External dependencies
 */
import { noop } from "lodash";
import isDeepEqual from "fast-deep-equal/react";

/**
 * WordPress dependencies
 */
import { __ } from "@wordpress/i18n";
import { lineDashed, lineDotted, lineSolid } from "@wordpress/icons";
import { __experimentalUnitControl as UnitControl } from "@wordpress/components";

/**
 * Internal dependencies
 */
import { LabelControl } from "../label-control";
import {
  BorderControlStyled,
  BorderGroupControlStyled,
  BorderStyleSelect,
} from "./styles";
import { ColorGradientDropdown } from "../color-gradient-dropdown";
import { getColorObject, useMultipleOriginColors } from "../../utils";

const BORDER_STYLES = [
  { name: __("-Style-"), key: "" },
  { name: __("None"), key: "none" },
  { name: lineSolid, key: "solid" },
  { name: lineDashed, key: "dashed" },
  { name: lineDotted, key: "dotted" },
];

/**
 * Border control: width, style, color
 *
 * @param {Object}
 * @returns
 */
export const BorderControl = ({
  label,
  values,
  onChange = noop,
  ...otherProps
}) => {
  const fields = [
    { name: "width", label: "" },
    { name: "style", label: "" },
    { name: "color", label: __("Color") },
  ];

  const { allColors } = useMultipleOriginColors();

  const renderControl = ({ name, label, value, onChange }) => {
    switch (name) {
      case "width":
        return (
          <UnitControl
            label={label}
            value={value}
            min={0}
            onChange={onChange}
            placeholder={__("Width")}
          />
        );

      case "style":
        return (
          <BorderStyleSelect
            label={label}
            value={BORDER_STYLES.find(({ key }) => key === value)}
            onChange={({ selectedItem: { key: value } }) => {
              onChange(value);
            }}
            options={BORDER_STYLES}
          />
        );

      case "color":
        return (
          <ColorGradientDropdown
            enableAlpha={true}
            settings={[
              {
                label,
                onColorChange: (value) => {
                  onChange(getColorObject(value, allColors));
                },
                colorValue: getColorObject(value, allColors)?.value,
              },
            ]}
            className="is-inner-control"
          />
        );

      default:
        break;
    }

    return null;
  };
  return (
    <>
      <BorderControlStyled
        label={label}
        fields={fields}
        values={values}
        renderLabel={({ label, isResponsive = false }) => {
          return (
            <LabelControl
              label={label}
              isResponsive={isResponsive}
              isBold={true}
            />
          );
        }}
        renderControl={renderControl}
        isLinkedGroup={false}
        onChange={onChange}
        columns={3}
        className="border-control"
        {...otherProps}
      />
    </>
  );
};

/**
 * Group of border: top, right, bottom, left
 * @param {Ojbect}
 */
export const BorderGroupControl = ({ label, values, onChange }) => {
  const fields = [
    { name: "top", label: __("Top") },
    { name: "right", label: __("Right") },
    { name: "bottom", label: __("Bottom") },
    { name: "left", label: __("Left") },
  ];
  return (
    <>
      <BorderGroupControlStyled
        fields={fields}
        values={values}
        renderLabel={() => {
          return <LabelControl label={label} isResponsive={true} />;
        }}
        renderControl={({ label, value, onChange }) => (
          <BorderControl values={value} label={label} onChange={onChange} />
        )}
        isLinkedGroup={true}
        getInitialLinkedState={({ top, right, bottom, left }) =>
          isDeepEqual(top, right) &&
          isDeepEqual(right, bottom) &&
          isDeepEqual(bottom, left)
        }
        onChange={onChange}
      />
    </>
  );
};
